@@ -1,8 +1,8 @@
 #!/bin/sh
 set -e
 tools/make_requests
-dlls/winevulkan/make_vulkan -x vk.xml
 tools/make_specfiles
+dlls/winevulkan/make_vulkan -x vk.xml -X video.xml
 autoreconf -ifv
 rm -rf autom4te.cache
 
