@@ -72,7 +72,10 @@ struct ctrl_handler
 static BOOL WINAPI default_ctrl_handler( DWORD type )
 {
     FIXME( "Terminating process %lx on event %lx\n", GetCurrentProcessId(), type );
-    RtlExitUserProcess( 0 );
+    if (type == CTRL_C_EVENT || type == CTRL_BREAK_EVENT)
+        RtlExitUserProcess( STATUS_CONTROL_C_EXIT );
+    else
+        RtlExitUserProcess( 0 );
     return TRUE;
 }
 
@@ -1211,11 +1214,8 @@ COORD WINAPI DECLSPEC_HOTPATCH GetLargestConsoleWindowSize( HANDLE handle )
  */
 BOOL WINAPI DECLSPEC_HOTPATCH GetNumberOfConsoleInputEvents( HANDLE handle, DWORD *count )
 {
-    struct condrv_input_info info;
-    if (!console_ioctl( handle, IOCTL_CONDRV_GET_INPUT_INFO, NULL, 0, &info, sizeof(info), NULL ))
-        return FALSE;
-    *count = info.input_count;
-    return TRUE;
+    return console_ioctl( handle, IOCTL_CONDRV_GET_INPUT_COUNT, NULL, 0,
+                          count, sizeof(*count), NULL );
 }
 
 
