@@ -1472,6 +1472,29 @@ DWORD WINAPI DECLSPEC_HOTPATCH GetModuleFileNameExA( HANDLE process, HMODULE mod
 }
 
 
+static NTSTATUS get_process_image_file_name( HANDLE process, BYTE *buffer, size_t buffer_size,
+                                             void **dynamic_buffer, UNICODE_STRING **result )
+{
+    NTSTATUS status;
+    DWORD needed;
+
+    /* FIXME: Use ProcessImageFileName for the PROCESS_NAME_NATIVE case */
+    status = NtQueryInformationProcess( process, ProcessImageFileNameWin32, buffer,
+                                        sizeof(buffer) - sizeof(WCHAR), &needed );
+    if (status == STATUS_INFO_LENGTH_MISMATCH)
+    {
+        *dynamic_buffer = HeapAlloc( GetProcessHeap(), 0, needed + sizeof(WCHAR) );
+        status = NtQueryInformationProcess( process, ProcessImageFileNameWin32, *dynamic_buffer,
+                                            needed, &needed );
+        if (status) HeapFree( GetProcessHeap(), 0, *dynamic_buffer );
+        *result = *dynamic_buffer;
+    }
+    else
+        *result = (UNICODE_STRING *)buffer;
+    return status;
+}
+
+
 /***********************************************************************
  *         GetModuleFileNameExW   (kernelbase.@)
  *         K32GetModuleFileNameExW   (kernelbase.@)
@@ -1484,29 +1507,47 @@ DWORD WINAPI DECLSPEC_HOTPATCH GetModuleFileNameExW( HANDLE process, HMODULE mod
 
     if (!size) return 0;
 
-    if (!IsWow64Process( process, &wow64 )) return 0;
-
-    if (is_win64 && wow64)
+    if (module)
     {
-        LDR_DATA_TABLE_ENTRY32 ldr_module32;
+        if (!IsWow64Process( process, &wow64 )) return 0;
 
-        if (get_ldr_module32( process, module, &ldr_module32 ))
+        if (is_win64 && wow64)
         {
-            len = ldr_module32.FullDllName.Length / sizeof(WCHAR);
-            if (ReadProcessMemory( process, (void *)(DWORD_PTR)ldr_module32.FullDllName.Buffer,
-                                   name, min( len, size ) * sizeof(WCHAR), NULL ))
-                found = TRUE;
+            LDR_DATA_TABLE_ENTRY32 ldr_module32;
+
+            if (get_ldr_module32( process, module, &ldr_module32 ))
+            {
+                len = ldr_module32.FullDllName.Length / sizeof(WCHAR);
+                if (ReadProcessMemory( process, (void *)(DWORD_PTR)ldr_module32.FullDllName.Buffer,
+                                       name, min( len, size ) * sizeof(WCHAR), NULL ))
+                    found = TRUE;
+            }
+        }
+        if (!found)
+        {
+            LDR_DATA_TABLE_ENTRY ldr_module;
+
+            if (!get_ldr_module(process, module, &ldr_module)) return 0;
+            len = ldr_module.FullDllName.Length / sizeof(WCHAR);
+            if (!ReadProcessMemory( process, ldr_module.FullDllName.Buffer,
+                                    name, min( len, size ) * sizeof(WCHAR), NULL ))
+                return 0;
         }
     }
-    if (!found)
+    else
     {
-        LDR_DATA_TABLE_ENTRY ldr_module;
+        BYTE buffer[sizeof(UNICODE_STRING) + MAX_PATH*sizeof(WCHAR)];  /* this buffer should be enough */
+        void *dynamic_buffer = NULL;
+        UNICODE_STRING *result;
+        NTSTATUS status;
 
-        if (!get_ldr_module(process, module, &ldr_module)) return 0;
-        len = ldr_module.FullDllName.Length / sizeof(WCHAR);
-        if (!ReadProcessMemory( process, ldr_module.FullDllName.Buffer,
-                                name, min( len, size ) * sizeof(WCHAR), NULL ))
-            return 0;
+        status = get_process_image_file_name( process, buffer, sizeof(buffer), &dynamic_buffer, &result );
+        if (set_ntstatus( status ))
+        {
+            len = result->Length / sizeof(WCHAR);
+            memcpy( name, result->Buffer, min( len, size - 1 ) * sizeof(WCHAR) );
+            HeapFree( GetProcessHeap(), 0, dynamic_buffer );
+        }
     }
 
     if (len < size)
@@ -1771,25 +1812,12 @@ BOOL WINAPI DECLSPEC_HOTPATCH QueryFullProcessImageNameW( HANDLE process, DWORD
                                                           WCHAR *name, DWORD *size )
 {
     BYTE buffer[sizeof(UNICODE_STRING) + MAX_PATH*sizeof(WCHAR)];  /* this buffer should be enough */
-    UNICODE_STRING *dynamic_buffer = NULL;
-    UNICODE_STRING *result = NULL;
+    void *dynamic_buffer = NULL;
+    UNICODE_STRING *result;
     NTSTATUS status;
-    DWORD needed;
 
-    /* FIXME: Use ProcessImageFileName for the PROCESS_NAME_NATIVE case */
-    status = NtQueryInformationProcess( process, ProcessImageFileNameWin32, buffer,
-                                        sizeof(buffer) - sizeof(WCHAR), &needed );
-    if (status == STATUS_INFO_LENGTH_MISMATCH)
-    {
-        dynamic_buffer = HeapAlloc( GetProcessHeap(), 0, needed + sizeof(WCHAR) );
-        status = NtQueryInformationProcess( process, ProcessImageFileNameWin32, dynamic_buffer,
-                                            needed, &needed );
-        result = dynamic_buffer;
-    }
-    else
-        result = (UNICODE_STRING *)buffer;
-
-    if (status) goto cleanup;
+    status = get_process_image_file_name( process, buffer, sizeof(buffer), &dynamic_buffer, &result );
+    if (status) return set_ntstatus( status );
 
     if (flags & PROCESS_NAME_NATIVE && result->Length > 2 * sizeof(WCHAR))
     {
