@@ -107,6 +107,75 @@ static inline struct audio_client *impl_from_IAudioStreamVolume(IAudioStreamVolu
     return CONTAINING_RECORD(iface, struct audio_client, IAudioStreamVolume_iface);
 }
 
+static HRESULT get_periods(struct audio_client *client,
+                           REFERENCE_TIME *def_period, REFERENCE_TIME *min_period)
+{
+    static const REFERENCE_TIME min_def_period = 100000; /* 10 ms */
+    struct get_device_period_params params;
+
+    params.device     = client->device_name;
+    params.flow       = client->dataflow;
+    params.def_period = def_period;
+    params.min_period = min_period;
+
+    wine_unix_call(get_device_period, &params);
+
+    if (def_period) *def_period = max(*def_period, min_def_period);
+
+    return params.result;
+}
+
+static HRESULT adjust_timing(struct audio_client *client, const BOOLEAN force_def_period,
+                             REFERENCE_TIME *duration, REFERENCE_TIME *period,
+                             const AUDCLNT_SHAREMODE mode, const DWORD flags,
+                             const WAVEFORMATEX *fmt)
+{
+    REFERENCE_TIME def_period, min_period;
+    HRESULT hr;
+
+    TRACE("Requested duration %lu and period %lu\n", (ULONG)*duration, (ULONG)*period);
+
+    if (FAILED(hr = get_periods(client, &def_period, &min_period)))
+        return hr;
+
+    TRACE("Device periods: %lu default and %lu minimum\n", (ULONG)def_period, (ULONG)min_period);
+
+    if (mode == AUDCLNT_SHAREMODE_SHARED) {
+        if (*period == 0 || force_def_period)
+            *period = def_period;
+        else if (*period < min_period)
+            return AUDCLNT_E_INVALID_DEVICE_PERIOD;
+        if (*duration < 3 * *period)
+            *duration = 3 * *period;
+    } else {
+        const WAVEFORMATEXTENSIBLE *fmtex = (WAVEFORMATEXTENSIBLE *)fmt;
+        if (fmtex->Format.wFormatTag == WAVE_FORMAT_EXTENSIBLE &&
+           (fmtex->dwChannelMask == 0 || fmtex->dwChannelMask & SPEAKER_RESERVED))
+            return AUDCLNT_E_UNSUPPORTED_FORMAT;
+        else {
+            if (*period == 0)
+                *period = def_period;
+            if (*period < min_period || *period > 5000000)
+                return AUDCLNT_E_INVALID_DEVICE_PERIOD;
+            else if (*duration > 20000000) /* The smaller the period, the lower this limit. */
+                return AUDCLNT_E_BUFFER_SIZE_ERROR;
+            else if (flags & AUDCLNT_STREAMFLAGS_EVENTCALLBACK) {
+                if (*duration != *period)
+                    return AUDCLNT_E_BUFDURATION_PERIOD_NOT_EQUAL;
+
+                FIXME("EXCLUSIVE mode with EVENTCALLBACK\n");
+
+                return AUDCLNT_E_DEVICE_IN_USE;
+            } else if (*duration < 8 * *period)
+                *duration = 8 * *period; /* May grow above 2s. */
+        }
+    }
+
+    TRACE("Adjusted duration %lu and period %lu\n", (ULONG)*duration, (ULONG)*period);
+
+    return hr;
+}
+
 static void dump_fmt(const WAVEFORMATEX *fmt)
 {
     TRACE("wFormatTag: 0x%x (", fmt->wFormatTag);
@@ -297,6 +366,134 @@ skip:
     return wcsdup(name);
 }
 
+static HRESULT stream_init(struct audio_client *client, const BOOLEAN force_def_period,
+                           const AUDCLNT_SHAREMODE mode, const DWORD flags,
+                           REFERENCE_TIME duration, REFERENCE_TIME period,
+                           const WAVEFORMATEX *fmt, const GUID *sessionguid)
+{
+    struct create_stream_params params;
+    UINT32 i, channel_count;
+    stream_handle stream;
+    WCHAR *name;
+
+    if (!fmt)
+        return E_POINTER;
+
+    dump_fmt(fmt);
+
+    if (mode != AUDCLNT_SHAREMODE_SHARED && mode != AUDCLNT_SHAREMODE_EXCLUSIVE)
+        return E_INVALIDARG;
+
+    if (flags & ~(AUDCLNT_STREAMFLAGS_CROSSPROCESS |
+                  AUDCLNT_STREAMFLAGS_LOOPBACK |
+                  AUDCLNT_STREAMFLAGS_EVENTCALLBACK |
+                  AUDCLNT_STREAMFLAGS_NOPERSIST |
+                  AUDCLNT_STREAMFLAGS_RATEADJUST |
+                  AUDCLNT_SESSIONFLAGS_EXPIREWHENUNOWNED |
+                  AUDCLNT_SESSIONFLAGS_DISPLAY_HIDE |
+                  AUDCLNT_SESSIONFLAGS_DISPLAY_HIDEWHENEXPIRED |
+                  AUDCLNT_STREAMFLAGS_SRC_DEFAULT_QUALITY |
+                  AUDCLNT_STREAMFLAGS_AUTOCONVERTPCM)) {
+        FIXME("Unknown flags: %08lx\n", flags);
+        return E_INVALIDARG;
+    }
+
+    sessions_lock();
+
+    if (client->stream) {
+        sessions_unlock();
+        return AUDCLNT_E_ALREADY_INITIALIZED;
+    }
+
+    if (FAILED(params.result = main_loop_start())) {
+        sessions_unlock();
+        return params.result;
+    }
+
+    if (flags & AUDCLNT_STREAMFLAGS_LOOPBACK) {
+        struct get_loopback_capture_device_params params;
+
+        if (client->dataflow != eRender) {
+            sessions_unlock();
+            return AUDCLNT_E_WRONG_ENDPOINT_TYPE;
+        }
+
+        params.device = client->device_name;
+        params.name = name = get_application_name();
+        params.ret_device_len = 0;
+        params.ret_device = NULL;
+        params.result = E_NOTIMPL;
+        wine_unix_call(get_loopback_capture_device, &params);
+        while (params.result == STATUS_BUFFER_TOO_SMALL) {
+            free(params.ret_device);
+            params.ret_device = malloc(params.ret_device_len);
+            wine_unix_call(get_loopback_capture_device, &params);
+        }
+        free(name);
+        if (FAILED(params.result)) {
+            sessions_unlock();
+            free(params.ret_device);
+            if (params.result == E_NOTIMPL)
+                FIXME("get_loopback_capture_device is not supported by backend.\n");
+            return params.result;
+        }
+        free(client->device_name);
+        client->device_name = params.ret_device;
+        client->dataflow = eCapture;
+    }
+
+    if (FAILED(params.result = adjust_timing(client, force_def_period, &duration, &period, mode, flags, fmt))) {
+        sessions_unlock();
+        return params.result;
+    }
+
+    params.name = name   = get_application_name();
+    params.device        = client->device_name;
+    params.flow          = client->dataflow;
+    params.share         = mode;
+    params.flags         = flags;
+    params.duration      = duration;
+    params.period        = period;
+    params.fmt           = fmt;
+    params.channel_count = &channel_count;
+    params.stream        = &stream;
+
+    wine_unix_call(create_stream, &params);
+
+    free(name);
+
+    if (FAILED(params.result)) {
+        sessions_unlock();
+        return params.result;
+    }
+
+    if (!(client->vols = malloc(channel_count * sizeof(*client->vols)))) {
+        params.result = E_OUTOFMEMORY;
+        goto exit;
+    }
+
+    for (i = 0; i < channel_count; i++)
+        client->vols[i] = 1.f;
+
+    params.result = get_audio_session(sessionguid, client->parent, channel_count, &client->session);
+
+exit:
+    if (FAILED(params.result)) {
+        stream_release(stream, NULL);
+        free(client->vols);
+        client->vols = NULL;
+    } else {
+        list_add_tail(&client->session->clients, &client->entry);
+        client->stream = stream;
+        client->channel_count = channel_count;
+        set_stream_volumes(client);
+    }
+
+    sessions_unlock();
+
+    return params.result;
+}
+
 static HRESULT WINAPI capture_QueryInterface(IAudioCaptureClient *iface, REFIID riid, void **ppv)
 {
     struct audio_client *This = impl_from_IAudioCaptureClient(iface);
@@ -484,130 +681,12 @@ static HRESULT WINAPI client_Initialize(IAudioClient3 *iface, AUDCLNT_SHAREMODE
                                  const WAVEFORMATEX *fmt, const GUID *sessionguid)
 {
     struct audio_client *This = impl_from_IAudioClient3(iface);
-    struct create_stream_params params;
-    UINT32 i, channel_count;
-    stream_handle stream;
-    WCHAR *name;
 
     TRACE("(%p)->(%x, %lx, %s, %s, %p, %s)\n", This, mode, flags, wine_dbgstr_longlong(duration),
                                                wine_dbgstr_longlong(period), fmt,
                                                debugstr_guid(sessionguid));
 
-    if (!fmt)
-        return E_POINTER;
-
-    dump_fmt(fmt);
-
-    if (mode != AUDCLNT_SHAREMODE_SHARED && mode != AUDCLNT_SHAREMODE_EXCLUSIVE)
-        return E_INVALIDARG;
-
-    if (flags & ~(AUDCLNT_STREAMFLAGS_CROSSPROCESS |
-                  AUDCLNT_STREAMFLAGS_LOOPBACK |
-                  AUDCLNT_STREAMFLAGS_EVENTCALLBACK |
-                  AUDCLNT_STREAMFLAGS_NOPERSIST |
-                  AUDCLNT_STREAMFLAGS_RATEADJUST |
-                  AUDCLNT_SESSIONFLAGS_EXPIREWHENUNOWNED |
-                  AUDCLNT_SESSIONFLAGS_DISPLAY_HIDE |
-                  AUDCLNT_SESSIONFLAGS_DISPLAY_HIDEWHENEXPIRED |
-                  AUDCLNT_STREAMFLAGS_SRC_DEFAULT_QUALITY |
-                  AUDCLNT_STREAMFLAGS_AUTOCONVERTPCM)) {
-        FIXME("Unknown flags: %08lx\n", flags);
-        return E_INVALIDARG;
-    }
-
-    sessions_lock();
-
-    if (This->stream) {
-        sessions_unlock();
-        return AUDCLNT_E_ALREADY_INITIALIZED;
-    }
-
-    if (FAILED(params.result = main_loop_start())) {
-        sessions_unlock();
-        return params.result;
-    }
-
-    if (flags & AUDCLNT_STREAMFLAGS_LOOPBACK)
-    {
-        struct get_loopback_capture_device_params params;
-
-        if (This->dataflow != eRender)
-        {
-            sessions_unlock();
-            return AUDCLNT_E_WRONG_ENDPOINT_TYPE;
-        }
-
-        params.device = This->device_name;
-        params.name = name = get_application_name();
-        params.ret_device_len = 0;
-        params.ret_device = NULL;
-        params.result = E_NOTIMPL;
-        wine_unix_call(get_loopback_capture_device, &params);
-        while (params.result == STATUS_BUFFER_TOO_SMALL)
-        {
-            free(params.ret_device);
-            params.ret_device = malloc(params.ret_device_len);
-            wine_unix_call(get_loopback_capture_device, &params);
-        }
-        free(name);
-        if (FAILED(params.result))
-        {
-            sessions_unlock();
-            free(params.ret_device);
-            if (params.result == E_NOTIMPL)
-                FIXME("get_loopback_capture_device is not supported by backend.\n");
-            return params.result;
-        }
-        free(This->device_name);
-        This->device_name = params.ret_device;
-        This->dataflow = eCapture;
-    }
-
-    params.name = name   = get_application_name();
-    params.device        = This->device_name;
-    params.flow          = This->dataflow;
-    params.share         = mode;
-    params.flags         = flags;
-    params.duration      = duration;
-    params.period        = period;
-    params.fmt           = fmt;
-    params.channel_count = &channel_count;
-    params.stream        = &stream;
-
-    wine_unix_call(create_stream, &params);
-
-    free(name);
-
-    if (FAILED(params.result)) {
-        sessions_unlock();
-        return params.result;
-    }
-
-    if (!(This->vols = malloc(channel_count * sizeof(*This->vols)))) {
-        params.result = E_OUTOFMEMORY;
-        goto exit;
-    }
-
-    for (i = 0; i < channel_count; i++)
-        This->vols[i] = 1.f;
-
-    params.result = get_audio_session(sessionguid, This->parent, channel_count, &This->session);
-
-exit:
-    if (FAILED(params.result)) {
-        stream_release(stream, NULL);
-        free(This->vols);
-        This->vols = NULL;
-    } else {
-        list_add_tail(&This->session->clients, &This->entry);
-        This->stream = stream;
-        This->channel_count = channel_count;
-        set_stream_volumes(This);
-    }
-
-    sessions_unlock();
-
-    return params.result;
+    return stream_init(This, TRUE, mode, flags, duration, period, fmt, sessionguid);
 }
 
 static HRESULT WINAPI client_GetBufferSize(IAudioClient3 *iface, UINT32 *out)
@@ -739,21 +818,13 @@ static HRESULT WINAPI client_GetDevicePeriod(IAudioClient3 *iface, REFERENCE_TIM
                                       REFERENCE_TIME *minperiod)
 {
     struct audio_client *This = impl_from_IAudioClient3(iface);
-    struct get_device_period_params params;
 
     TRACE("(%p)->(%p, %p)\n", This, defperiod, minperiod);
 
     if (!defperiod && !minperiod)
         return E_POINTER;
 
-    params.device     = This->device_name;
-    params.flow       = This->dataflow;
-    params.def_period = defperiod;
-    params.min_period = minperiod;
-
-    wine_unix_call(get_device_period, &params);
-
-    return params.result;
+    return get_periods(This, defperiod, minperiod);
 }
 
 static HRESULT WINAPI client_Start(IAudioClient3 *iface)
@@ -904,6 +975,9 @@ static HRESULT WINAPI client_GetService(IAudioClient3 *iface, REFIID riid, void
 
         if (!new_session)
             IUnknown_AddRef((IUnknown *)*ppv);
+    } else if (IsEqualIID(riid, &IID_IAudioClockAdjustment)) {
+        IAudioClockAdjustment_AddRef(&This->IAudioClockAdjustment_iface);
+        *ppv = &This->IAudioClockAdjustment_iface;
     } else {
             FIXME("stub %s\n", debugstr_guid(riid));
             hr = E_NOINTERFACE;
@@ -978,10 +1052,24 @@ static HRESULT WINAPI client_GetSharedModeEnginePeriod(IAudioClient3 *iface,
                                                 UINT32 *max_period_frames)
 {
     struct audio_client *This = impl_from_IAudioClient3(iface);
-    FIXME("(%p)->(%p, %p, %p, %p, %p) - stub\n", This, format, default_period_frames,
-                                                 unit_period_frames, min_period_frames,
-                                                 max_period_frames);
-    return E_NOTIMPL;
+    REFERENCE_TIME def_period, min_period;
+    HRESULT hr;
+
+    TRACE("(%p)->(%p, %p, %p, %p, %p)\n",
+          This, format, default_period_frames,
+          unit_period_frames, min_period_frames,
+          max_period_frames);
+
+    if (FAILED(hr = get_periods(This, &def_period, &min_period)))
+        return hr;
+
+    *default_period_frames = def_period * format->nSamplesPerSec / (REFERENCE_TIME)10000000;
+    *min_period_frames     = (min_period * format->nSamplesPerSec + 10000000 - 1) / (REFERENCE_TIME)10000000;
+    *default_period_frames = max( *default_period_frames, *min_period_frames );
+    *max_period_frames     = *default_period_frames;
+    *unit_period_frames    = 1;
+
+    return hr;
 }
 
 static HRESULT WINAPI client_GetCurrentSharedModeEnginePeriod(IAudioClient3 *iface,
@@ -989,8 +1077,18 @@ static HRESULT WINAPI client_GetCurrentSharedModeEnginePeriod(IAudioClient3 *ifa
                                                        UINT32 *cur_period_frames)
 {
     struct audio_client *This = impl_from_IAudioClient3(iface);
-    FIXME("(%p)->(%p, %p) - stub\n", This, cur_format, cur_period_frames);
-    return E_NOTIMPL;
+    UINT32 dummy;
+    HRESULT hr;
+
+    TRACE("(%p)->(%p, %p)\n", This, cur_format, cur_period_frames);
+
+    if (!cur_format || !cur_period_frames)
+        return E_POINTER;
+
+    if (FAILED(hr = client_GetMixFormat(iface, cur_format)))
+        return hr;
+
+    return client_GetSharedModeEnginePeriod(iface, *cur_format, cur_period_frames, &dummy, &dummy, &dummy);
 }
 
 static HRESULT WINAPI client_InitializeSharedAudioStream(IAudioClient3 *iface, DWORD flags,
@@ -999,8 +1097,16 @@ static HRESULT WINAPI client_InitializeSharedAudioStream(IAudioClient3 *iface, D
                                                   const GUID *session_guid)
 {
     struct audio_client *This = impl_from_IAudioClient3(iface);
-    FIXME("(%p)->(0x%lx, %u, %p, %s) - stub\n", This, flags, period_frames, format, debugstr_guid(session_guid));
-    return E_NOTIMPL;
+    REFERENCE_TIME period;
+
+    TRACE("(%p)->(0x%lx, %u, %p, %s)\n", This, flags, period_frames, format, debugstr_guid(session_guid));
+
+    if (!format)
+        return E_POINTER;
+
+    period = period_frames * (REFERENCE_TIME)10000000 / format->nSamplesPerSec;
+
+    return stream_init(This, FALSE, AUDCLNT_SHAREMODE_SHARED, flags, 0, period, format, session_guid);
 }
 
 const IAudioClient3Vtbl AudioClient3_Vtbl =
