@@ -32,6 +32,7 @@ static NTSTATUS (WINAPI *pRtlMultiByteToUnicodeN)( LPWSTR dst, DWORD dstlen, LPD
                                                    LPCSTR src, DWORD srclen );
 static NTSTATUS (WINAPI *pRtlQueryEnvironmentVariable_U)(PWSTR, PUNICODE_STRING, PUNICODE_STRING);
 static NTSTATUS (WINAPI* pRtlQueryEnvironmentVariable)(WCHAR*, WCHAR*, SIZE_T, WCHAR*, SIZE_T, SIZE_T*);
+static NTSTATUS (WINAPI *pRtlExpandEnvironmentStrings)(WCHAR*, WCHAR*, SIZE_T, WCHAR*, SIZE_T, SIZE_T*);
 static NTSTATUS (WINAPI *pRtlExpandEnvironmentStrings_U)(LPWSTR, PUNICODE_STRING, PUNICODE_STRING, PULONG);
 static NTSTATUS (WINAPI *pRtlCreateProcessParameters)(RTL_USER_PROCESS_PARAMETERS**,
                                                       const UNICODE_STRING*, const UNICODE_STRING*,
@@ -243,6 +244,113 @@ static void testExpand(void)
 
 }
 
+static void test_RtlExpandEnvironmentStrings(void)
+{
+    int i;
+    WCHAR buf[256];
+    HRESULT status;
+    UNICODE_STRING us_src, us_dst, us_name, us_value;
+    static const struct test_info
+    {
+        const WCHAR *input;
+        const WCHAR *expected_str;
+        int count_in;
+        int expected_count_out;
+    } tests[] =
+    {
+        /*  0 */ { L"Long long value",       L"abcdefghijklmnopqrstuv",  0, 16 },
+        /*  1 */ { L"Long long value",       L"abcdefghijklmnopqrstuv",  1, 16 },
+        /*  2 */ { L"Long long value",       L"Lbcdefghijklmnopqrstuv",  2, 16 },
+        /*  3 */ { L"Long long value",       L"Locdefghijklmnopqrstuv",  3, 16 },
+        /*  4 */ { L"Long long value",       L"Long long valuopqrstuv", 15, 16 },
+        /*  5 */ { L"Long long value",       L"Long long value",        16, 16 },
+        /*  6 */ { L"Long long value",       L"Long long value",        17, 16 },
+        /*  7 */ { L"%TVAR% long long",      L"abcdefghijklmnopqrstuv",  0, 15 },
+        /*  8 */ { L"%TVAR% long long",      L"",                        1, 15 },
+        /*  9 */ { L"%TVAR% long long",      L"",                        2, 15 },
+        /* 10 */ { L"%TVAR% long long",      L"",                        4, 15 },
+        /* 11 */ { L"%TVAR% long long",      L"WINE",                    5, 15 },
+        /* 12 */ { L"%TVAR% long long",      L"WINE fghijklmnopqrstuv",  6, 15 },
+        /* 13 */ { L"%TVAR% long long",      L"WINE lghijklmnopqrstuv",  7, 15 },
+        /* 14 */ { L"%TVAR% long long",      L"WINE long long",         15, 15 },
+        /* 15 */ { L"%TVAR% long long",      L"WINE long long",         16, 15 },
+        /* 16 */ { L"%TVAR%%TVAR% long",     L"",                        4, 14 },
+        /* 17 */ { L"%TVAR%%TVAR% long",     L"WINE",                    5, 14 },
+        /* 18 */ { L"%TVAR%%TVAR% long",     L"WINE",                    6, 14 },
+        /* 19 */ { L"%TVAR%%TVAR% long",     L"WINE",                    8, 14 },
+        /* 20 */ { L"%TVAR%%TVAR% long",     L"WINEWINE",                9, 14 },
+        /* 21 */ { L"%TVAR%%TVAR% long",     L"WINEWINE jklmnopqrstuv", 10, 14 },
+        /* 22 */ { L"%TVAR%%TVAR% long",     L"WINEWINE long",          14, 14 },
+        /* 23 */ { L"%TVAR%%TVAR% long",     L"WINEWINE long",          15, 14 },
+        /* 24 */ { L"%TVAR% %TVAR% long",    L"WINE",                    5, 15 },
+        /* 25 */ { L"%TVAR% %TVAR% long",    L"WINE ",                   6, 15 },
+        /* 26 */ { L"%TVAR% %TVAR% long",    L"WINE ",                   8, 15 },
+        /* 27 */ { L"%TVAR% %TVAR% long",    L"WINE ",                   9, 15 },
+        /* 28 */ { L"%TVAR% %TVAR% long",    L"WINE WINE",              10, 15 },
+        /* 29 */ { L"%TVAR% %TVAR% long",    L"WINE WINE klmnopqrstuv", 11, 15 },
+        /* 30 */ { L"%TVAR% %TVAR% long",    L"WINE WINE llmnopqrstuv", 12, 15 },
+        /* 31 */ { L"%TVAR% %TVAR% long",    L"WINE WINE lonnopqrstuv", 14, 15 },
+        /* 32 */ { L"%TVAR% %TVAR% long",    L"WINE WINE long",         15, 15 },
+        /* 33 */ { L"%TVAR% %TVAR% long",    L"WINE WINE long",         16, 15 },
+        /* 34 */ { L"%TVAR2% long long",     L"abcdefghijklmnopqrstuv",  1, 18 },
+        /* 35 */ { L"%TVAR2% long long",     L"%bcdefghijklmnopqrstuv",  2, 18 },
+        /* 36 */ { L"%TVAR2% long long",     L"%TVdefghijklmnopqrstuv",  4, 18 },
+        /* 37 */ { L"%TVAR2% long long",     L"%TVAR2ghijklmnopqrstuv",  7, 18 },
+        /* 38 */ { L"%TVAR2% long long",     L"%TVAR2%hijklmnopqrstuv",  8, 18 },
+        /* 39 */ { L"%TVAR2% long long",     L"%TVAR2% ijklmnopqrstuv",  9, 18 },
+        /* 40 */ { L"%TVAR2% long long",     L"%TVAR2% ljklmnopqrstuv", 10, 18 },
+        /* 41 */ { L"%TVAR2% long long",     L"%TVAR2% long long",      18, 18 },
+        /* 42 */ { L"%TVAR2% long long",     L"%TVAR2% long long",      19, 18 },
+        /* 43 */ { L"%TVAR long long",       L"abcdefghijklmnopqrstuv",  1, 16 },
+        /* 44 */ { L"%TVAR long long",       L"%bcdefghijklmnopqrstuv",  2, 16 },
+        /* 45 */ { L"%TVAR long long",       L"%Tcdefghijklmnopqrstuv",  3, 16 },
+        /* 46 */ { L"%TVAR long long",       L"%TVAR long lonopqrstuv", 15, 16 },
+        /* 47 */ { L"%TVAR long long",       L"%TVAR long long",        16, 16 },
+        /* 48 */ { L"%TVAR long long",       L"%TVAR long long",        17, 16 },
+    };
+
+    RtlInitUnicodeString(&us_name, L"TVAR");
+    RtlInitUnicodeString(&us_value, L"WINE");
+    status = RtlSetEnvironmentVariable(NULL, &us_name, &us_value);
+    ok(status == STATUS_SUCCESS, "RtlSetEnvironmentVariable failed with %lx\n", status);
+
+    for (i = 0; i < ARRAY_SIZE(tests); i++)
+    {
+        const struct test_info *test = &tests[i];
+        SIZE_T out_len;
+        HRESULT expected_status = test->count_in >= test->expected_count_out ? STATUS_SUCCESS : STATUS_BUFFER_TOO_SMALL;
+
+        wcscpy(buf, L"abcdefghijklmnopqrstuv");
+        status = pRtlExpandEnvironmentStrings(NULL, (WCHAR*)test->input, wcslen(test->input), buf, test->count_in, &out_len);
+        ok(out_len == test->expected_count_out, "Test %d: got %Iu\n", i, out_len);
+        ok(status == expected_status, "Test %d: Expected status %lx, got %lx\n", i, expected_status, status);
+        ok(!wcscmp(buf, test->expected_str), "Test %d: got %s\n", i, debugstr_w(buf));
+    }
+
+    for (i = 0; i < ARRAY_SIZE(tests); i++)
+    {
+        const struct test_info *test = &tests[i];
+        DWORD out_len;
+        HRESULT expected_status = test->count_in >= test->expected_count_out ? STATUS_SUCCESS : STATUS_BUFFER_TOO_SMALL;
+
+        us_src.Length = wcslen(test->input) * sizeof(WCHAR);
+        us_src.MaximumLength = us_src.Length;
+        us_src.Buffer = (WCHAR*)test->input;
+
+        us_dst.Length = test->count_in * sizeof(WCHAR);
+        us_dst.MaximumLength = us_dst.Length;
+        us_dst.Buffer = buf;
+
+        wcscpy(buf, L"abcdefghijklmnopqrstuv");
+        status = pRtlExpandEnvironmentStrings_U(NULL, &us_src, &us_dst, &out_len);
+        ok(out_len / sizeof(WCHAR) == test->expected_count_out, "Test %d: got %lu\n", i, out_len);
+        ok(status == expected_status, "Test %d: Expected status %lx, got %lx\n", i, expected_status, status);
+        ok(!wcscmp(buf, test->expected_str), "Test %d: got %s\n", i, debugstr_w(buf));
+    }
+    status = RtlSetEnvironmentVariable(NULL, &us_name, NULL);
+    ok(status == STATUS_SUCCESS, "RtlSetEnvironmentVariable failed with %lx\n", status);
+}
+
 static WCHAR *get_params_string( RTL_USER_PROCESS_PARAMETERS *params, UNICODE_STRING *str )
 {
     if (params->Flags & PROCESS_PARAMS_FLAG_NORMALIZED) return str->Buffer;
@@ -621,6 +729,7 @@ START_TEST(env)
     pRtlMultiByteToUnicodeN = (void *)GetProcAddress(mod,"RtlMultiByteToUnicodeN");
     pRtlQueryEnvironmentVariable_U = (void*)GetProcAddress(mod, "RtlQueryEnvironmentVariable_U");
     pRtlQueryEnvironmentVariable = (void*)GetProcAddress(mod, "RtlQueryEnvironmentVariable");
+    pRtlExpandEnvironmentStrings = (void*)GetProcAddress(mod, "RtlExpandEnvironmentStrings");
     pRtlExpandEnvironmentStrings_U = (void*)GetProcAddress(mod, "RtlExpandEnvironmentStrings_U");
     pRtlCreateProcessParameters = (void*)GetProcAddress(mod, "RtlCreateProcessParameters");
     pRtlDestroyProcessParameters = (void*)GetProcAddress(mod, "RtlDestroyProcessParameters");
@@ -630,4 +739,5 @@ START_TEST(env)
     test_process_params();
     test_RtlSetCurrentEnvironment();
     test_RtlSetEnvironmentVariable();
+    test_RtlExpandEnvironmentStrings();
 }
