@@ -2516,14 +2516,3 @@ void WINAPI RtlSetLastWin32Error( DWORD err )
 #endif
     teb->LastErrorValue = err;
 }
-
-
-/**********************************************************************
- *      __wine_set_unix_env  (ntdll.so)
- */
-NTSTATUS WINAPI __wine_set_unix_env( const char *var, const char *val )
-{
-    if (!val) unsetenv(var);
-    else setenv(var, val, 1);
-    return 0;
-}
