@@ -25,6 +25,7 @@
 #include "mftransform.h"
 #include "wmcodecdsp.h"
 #include "mediaerr.h"
+#include "dmort.h"
 
 #include "wine/debug.h"
 
@@ -55,8 +56,8 @@ struct wma_decoder
     IUnknown *outer;
     LONG refcount;
 
-    struct wg_format input_format;
-    struct wg_format output_format;
+    DMO_MEDIA_TYPE input_type;
+    DMO_MEDIA_TYPE output_type;
 
     DWORD input_buf_size;
     DWORD output_buf_size;
@@ -75,19 +76,13 @@ static HRESULT try_create_wg_transform(struct wma_decoder *decoder)
     struct wg_transform_attrs attrs = {0};
 
     if (decoder->wg_transform)
+    {
         wg_transform_destroy(decoder->wg_transform);
-    decoder->wg_transform = 0;
+        decoder->wg_transform = 0;
+    }
 
-    if (decoder->input_format.major_type == WG_MAJOR_TYPE_UNKNOWN)
-        return MF_E_INVALIDMEDIATYPE;
-
-    if (decoder->output_format.major_type == WG_MAJOR_TYPE_UNKNOWN)
-        return MF_E_INVALIDMEDIATYPE;
-
-    if (!(decoder->wg_transform = wg_transform_create(&decoder->input_format, &decoder->output_format, &attrs)))
-        return E_FAIL;
-
-    return S_OK;
+    return wg_transform_create_quartz(&decoder->input_type, &decoder->output_type,
+            &attrs, &decoder->wg_transform);
 }
 
 static HRESULT WINAPI unknown_QueryInterface(IUnknown *iface, REFIID iid, void **out)
@@ -138,6 +133,9 @@ static ULONG WINAPI unknown_Release(IUnknown *iface)
             wg_transform_destroy(decoder->wg_transform);
 
         wg_sample_queue_destroy(decoder->wg_sample_queue);
+
+        MoFreeMediaType(&decoder->input_type);
+        MoFreeMediaType(&decoder->output_type);
         free(decoder);
     }
 
@@ -204,8 +202,8 @@ static HRESULT WINAPI transform_GetInputStreamInfo(IMFTransform *iface, DWORD id
 
     TRACE("iface %p, id %lu, info %p.\n", iface, id, info);
 
-    if (decoder->input_format.major_type == WG_MAJOR_TYPE_UNKNOWN
-            || decoder->output_format.major_type == WG_MAJOR_TYPE_UNKNOWN)
+    if (IsEqualGUID(&decoder->input_type.majortype, &GUID_NULL)
+            || IsEqualGUID(&decoder->output_type.majortype, &GUID_NULL))
     {
         memset(info, 0, sizeof(*info));
         return MF_E_TRANSFORM_TYPE_NOT_SET;
@@ -225,8 +223,8 @@ static HRESULT WINAPI transform_GetOutputStreamInfo(IMFTransform *iface, DWORD i
 
     TRACE("iface %p, id %lu, info %p.\n", iface, id, info);
 
-    if (decoder->input_format.major_type == WG_MAJOR_TYPE_UNKNOWN
-            || decoder->output_format.major_type == WG_MAJOR_TYPE_UNKNOWN)
+    if (IsEqualGUID(&decoder->input_type.majortype, &GUID_NULL)
+            || IsEqualGUID(&decoder->output_type.majortype, &GUID_NULL))
     {
         memset(info, 0, sizeof(*info));
         return MF_E_TRANSFORM_TYPE_NOT_SET;
@@ -282,11 +280,12 @@ static HRESULT WINAPI transform_GetOutputAvailableType(IMFTransform *iface, DWOR
     struct wma_decoder *decoder = impl_from_IMFTransform(iface);
     IMFMediaType *media_type;
     const GUID *output_type;
+    WAVEFORMATEX *wfx;
     HRESULT hr;
 
     TRACE("iface %p, id %lu, index %lu, type %p.\n", iface, id, index, type);
 
-    if (decoder->input_format.major_type == WG_MAJOR_TYPE_UNKNOWN)
+    if (IsEqualGUID(&decoder->input_type.majortype, &GUID_NULL))
         return MF_E_TRANSFORM_TYPE_NOT_SET;
 
     *type = NULL;
@@ -318,20 +317,16 @@ static HRESULT WINAPI transform_GetOutputAvailableType(IMFTransform *iface, DWOR
             sample_size)))
         goto done;
 
-    if (FAILED(hr = IMFMediaType_SetUINT32(media_type, &MF_MT_AUDIO_NUM_CHANNELS,
-            decoder->input_format.u.audio_wma.channels)))
+    wfx = (WAVEFORMATEX *)decoder->input_type.pbFormat;
+    if (FAILED(hr = IMFMediaType_SetUINT32(media_type, &MF_MT_AUDIO_NUM_CHANNELS, wfx->nChannels)))
+        goto done;
+    if (FAILED(hr = IMFMediaType_SetUINT32(media_type, &MF_MT_AUDIO_SAMPLES_PER_SECOND, wfx->nSamplesPerSec)))
         goto done;
 
-    if (FAILED(hr = IMFMediaType_SetUINT32(media_type, &MF_MT_AUDIO_SAMPLES_PER_SECOND,
-            decoder->input_format.u.audio_wma.rate)))
+    block_alignment = sample_size * wfx->nChannels / 8;
+    if (FAILED(hr = IMFMediaType_SetUINT32(media_type, &MF_MT_AUDIO_BLOCK_ALIGNMENT, block_alignment)))
         goto done;
-
-    block_alignment = sample_size * decoder->input_format.u.audio_wma.channels / 8;
-    if (FAILED(hr = IMFMediaType_SetUINT32(media_type, &MF_MT_AUDIO_BLOCK_ALIGNMENT,
-            block_alignment)))
-        goto done;
-    if (FAILED(hr = IMFMediaType_SetUINT32(media_type, &MF_MT_AUDIO_AVG_BYTES_PER_SECOND,
-            decoder->input_format.u.audio_wma.rate * block_alignment)))
+    if (FAILED(hr = IMFMediaType_SetUINT32(media_type, &MF_MT_AUDIO_AVG_BYTES_PER_SECOND, wfx->nSamplesPerSec * block_alignment)))
         goto done;
 
     if (FAILED(hr = IMFMediaType_SetUINT32(media_type, &MF_MT_ALL_SAMPLES_INDEPENDENT, 1)))
@@ -387,9 +382,13 @@ static HRESULT WINAPI transform_SetInputType(IMFTransform *iface, DWORD id, IMFM
     if (flags & MFT_SET_TYPE_TEST_ONLY)
         return S_OK;
 
-    mf_media_type_to_wg_format(type, &decoder->input_format);
-    decoder->input_buf_size = block_alignment;
-    decoder->output_format.major_type = WG_MAJOR_TYPE_UNKNOWN;
+    MoFreeMediaType(&decoder->output_type);
+    memset(&decoder->output_type, 0, sizeof(decoder->output_type));
+    MoFreeMediaType(&decoder->input_type);
+    memset(&decoder->input_type, 0, sizeof(decoder->input_type));
+
+    if (SUCCEEDED(hr = MFInitAMMediaTypeFromMFMediaType(type, GUID_NULL, &decoder->input_type)))
+        decoder->input_buf_size = block_alignment;
 
     return hr;
 }
@@ -405,7 +404,7 @@ static HRESULT WINAPI transform_SetOutputType(IMFTransform *iface, DWORD id, IMF
 
     TRACE("iface %p, id %lu, type %p, flags %#lx.\n", iface, id, type, flags);
 
-    if (decoder->input_format.major_type == WG_MAJOR_TYPE_UNKNOWN)
+    if (IsEqualGUID(&decoder->input_type.majortype, &GUID_NULL))
         return MF_E_TRANSFORM_TYPE_NOT_SET;
 
     if (FAILED(hr = IMFMediaType_GetGUID(type, &MF_MT_MAJOR_TYPE, &major)) ||
@@ -448,10 +447,15 @@ static HRESULT WINAPI transform_SetOutputType(IMFTransform *iface, DWORD id, IMF
     if (flags & MFT_SET_TYPE_TEST_ONLY)
         return S_OK;
 
-    decoder->input_format.u.audio_wma.depth = sample_size;
+    MoFreeMediaType(&decoder->output_type);
+    memset(&decoder->output_type, 0, sizeof(decoder->output_type));
 
-    mf_media_type_to_wg_format(type, &decoder->output_format);
-    decoder->output_buf_size = 1024 * block_alignment * channel_count;
+    if (SUCCEEDED(hr = MFInitAMMediaTypeFromMFMediaType(type, GUID_NULL, &decoder->output_type)))
+    {
+        WAVEFORMATEX *wfx = (WAVEFORMATEX *)decoder->input_type.pbFormat;
+        wfx->wBitsPerSample = sample_size;
+        decoder->output_buf_size = 1024 * block_alignment * channel_count;
+    }
 
     if (FAILED(hr = try_create_wg_transform(decoder)))
         goto failed;
@@ -459,7 +463,8 @@ static HRESULT WINAPI transform_SetOutputType(IMFTransform *iface, DWORD id, IMF
     return S_OK;
 
 failed:
-    decoder->output_format.major_type = WG_MAJOR_TYPE_UNKNOWN;
+    MoFreeMediaType(&decoder->output_type);
+    memset(&decoder->output_type, 0, sizeof(decoder->output_type));
     return hr;
 }
 
@@ -501,7 +506,18 @@ static HRESULT WINAPI transform_ProcessEvent(IMFTransform *iface, DWORD id, IMFM
 
 static HRESULT WINAPI transform_ProcessMessage(IMFTransform *iface, MFT_MESSAGE_TYPE message, ULONG_PTR param)
 {
-    FIXME("iface %p, message %#x, param %p stub!\n", iface, message, (void *)param);
+    struct wma_decoder *decoder = impl_from_IMFTransform(iface);
+
+    TRACE("iface %p, message %#x, param %p.\n", iface, message, (void *)param);
+
+    if (!decoder->wg_transform)
+        return MF_E_TRANSFORM_TYPE_NOT_SET;
+
+    if (message == MFT_MESSAGE_COMMAND_DRAIN)
+        return wg_transform_drain(decoder->wg_transform);
+
+    FIXME("Ignoring message %#x.\n", message);
+
     return S_OK;
 }
 
@@ -554,7 +570,7 @@ static HRESULT WINAPI transform_ProcessOutput(IMFTransform *iface, DWORD flags,
         return hr;
 
     if (SUCCEEDED(hr = wg_transform_read_mf(decoder->wg_transform, samples->pSample,
-            info.cbSize, NULL, &samples->dwStatus)))
+            info.cbSize, &samples->dwStatus, NULL)))
         wg_sample_queue_flush(decoder->wg_sample_queue, false);
 
     return hr;
@@ -658,7 +674,9 @@ static HRESULT WINAPI media_object_GetOutputType(IMediaObject *iface, DWORD inde
         DMO_MEDIA_TYPE *type)
 {
     struct wma_decoder *decoder = impl_from_IMediaObject(iface);
-    WAVEFORMATEX *wfx;
+    UINT32 depth, channels, rate;
+    IMFMediaType *media_type;
+    HRESULT hr;
 
     TRACE("iface %p, index %lu, type_index %lu, type %p\n", iface, index, type_index, type);
 
@@ -666,42 +684,44 @@ static HRESULT WINAPI media_object_GetOutputType(IMediaObject *iface, DWORD inde
         return DMO_E_INVALIDSTREAMINDEX;
     if (type_index >= 1)
         return DMO_E_NO_MORE_ITEMS;
-    if (decoder->input_format.major_type == WG_MAJOR_TYPE_UNKNOWN)
+    if (IsEqualGUID(&decoder->input_type.majortype, &GUID_NULL))
         return DMO_E_TYPE_NOT_SET;
     if (!type)
         return S_OK;
 
-    memset(type, 0, sizeof(*type));
-    type->majortype = MFMediaType_Audio;
-    type->subtype = MEDIASUBTYPE_PCM;
-    type->formattype = FORMAT_WaveFormatEx;
-    type->bFixedSizeSamples = FALSE;
-    type->bTemporalCompression = TRUE;
-    type->lSampleSize = 0;
+    if (FAILED(hr = MFCreateMediaTypeFromRepresentation(AM_MEDIA_TYPE_REPRESENTATION,
+            &decoder->input_type, &media_type)))
+        return hr;
 
-    type->cbFormat = sizeof(WAVEFORMATEX);
-    type->pbFormat = CoTaskMemAlloc(type->cbFormat);
-    memset(type->pbFormat, 0, type->cbFormat);
-
-    wfx = (WAVEFORMATEX *)type->pbFormat;
-    if (decoder->input_format.u.audio_wma.depth == 32)
-        wfx->wFormatTag = WAVE_FORMAT_IEEE_FLOAT;
+    if (SUCCEEDED(IMFMediaType_GetUINT32(media_type, &MF_MT_AUDIO_BITS_PER_SAMPLE, &depth))
+            && depth == 32)
+        hr = IMFMediaType_SetGUID(media_type, &MF_MT_SUBTYPE, &MFAudioFormat_Float);
     else
-        wfx->wFormatTag = WAVE_FORMAT_PCM;
-    wfx->nChannels = decoder->input_format.u.audio_wma.channels;
-    wfx->nSamplesPerSec = decoder->input_format.u.audio_wma.rate;
-    wfx->wBitsPerSample = decoder->input_format.u.audio_wma.depth;
-    wfx->nAvgBytesPerSec = wfx->nChannels * wfx->nSamplesPerSec * wfx->wBitsPerSample / 8;
-    wfx->nBlockAlign = wfx->nChannels * wfx->wBitsPerSample / 8;
+        hr = IMFMediaType_SetGUID(media_type, &MF_MT_SUBTYPE, &MFAudioFormat_PCM);
 
-    return S_OK;
+    if (SUCCEEDED(hr))
+        hr = IMFMediaType_GetUINT32(media_type, &MF_MT_AUDIO_NUM_CHANNELS, &channels);
+    if (SUCCEEDED(hr))
+        hr = IMFMediaType_SetUINT32(media_type, &MF_MT_AUDIO_BLOCK_ALIGNMENT, depth * channels / 8);
+
+    if (SUCCEEDED(hr))
+        hr = IMFMediaType_GetUINT32(media_type, &MF_MT_AUDIO_SAMPLES_PER_SECOND, &rate);
+    if (SUCCEEDED(hr))
+        hr = IMFMediaType_SetUINT32(media_type, &MF_MT_AUDIO_AVG_BYTES_PER_SECOND, depth * channels / 8 * rate);
+
+    if (SUCCEEDED(hr))
+        hr = IMFMediaType_DeleteItem(media_type, &MF_MT_USER_DATA);
+    if (SUCCEEDED(hr))
+        hr = MFInitAMMediaTypeFromMFMediaType(media_type, GUID_NULL, type);
+
+    IMFMediaType_Release(media_type);
+    return hr;
 }
 
 static HRESULT WINAPI media_object_SetInputType(IMediaObject *iface, DWORD index,
         const DMO_MEDIA_TYPE *type, DWORD flags)
 {
     struct wma_decoder *decoder = impl_from_IMediaObject(iface);
-    struct wg_format wg_format;
 
     TRACE("iface %p, index %lu, type %p, flags %#lx.\n", iface, index, type, flags);
 
@@ -712,7 +732,8 @@ static HRESULT WINAPI media_object_SetInputType(IMediaObject *iface, DWORD index
     {
         if (flags != DMO_SET_TYPEF_CLEAR)
             return E_INVALIDARG;
-        memset(&decoder->input_format, 0, sizeof(decoder->input_format));
+        MoFreeMediaType(&decoder->input_type);
+        memset(&decoder->input_type, 0, sizeof(decoder->input_type));
         if (decoder->wg_transform)
         {
             wg_transform_destroy(decoder->wg_transform);
@@ -728,19 +749,35 @@ static HRESULT WINAPI media_object_SetInputType(IMediaObject *iface, DWORD index
     if (!IsEqualGUID(&type->majortype, &MEDIATYPE_Audio))
         return DMO_E_TYPE_NOT_ACCEPTED;
 
-    if (!amt_to_wg_format((const AM_MEDIA_TYPE *)type, &wg_format))
-        return DMO_E_TYPE_NOT_ACCEPTED;
-
     if (flags & DMO_SET_TYPEF_TEST_ONLY)
         return S_OK;
 
-    decoder->input_format = wg_format;
+    MoFreeMediaType(&decoder->input_type);
+    memset(&decoder->input_type, 0, sizeof(decoder->input_type));
+    MoCopyMediaType(&decoder->input_type, type);
+
     if (decoder->wg_transform)
     {
         wg_transform_destroy(decoder->wg_transform);
         decoder->wg_transform = 0;
     }
 
+{
+    const char *sgi = getenv("SteamGameId");
+    if (sgi && (0
+        || !strcmp(sgi, "802870")
+        || !strcmp(sgi, "1230140")
+        || !strcmp(sgi, "2515070")
+        || !strcmp(sgi, "3625380")
+    ))
+    {
+        WAVEFORMATEX *wfx = (WAVEFORMATEX *)decoder->input_type.pbFormat;
+        decoder->input_type.subtype = MFAudioFormat_PCM;
+        wfx->wFormatTag = WAVE_FORMAT_PCM;
+        wfx->wBitsPerSample = 16;
+    }
+}
+
     return S_OK;
 }
 
@@ -749,8 +786,8 @@ static HRESULT WINAPI media_object_SetOutputType(IMediaObject *iface, DWORD inde
 {
     struct wma_decoder *decoder = impl_from_IMediaObject(iface);
     struct wg_transform_attrs attrs = {0};
-    struct wg_format wg_format;
     unsigned int i;
+    HRESULT hr;
 
     TRACE("iface %p, index %lu, type %p, flags %#lx,\n", iface, index, type, flags);
 
@@ -761,7 +798,8 @@ static HRESULT WINAPI media_object_SetOutputType(IMediaObject *iface, DWORD inde
     {
         if (flags != DMO_SET_TYPEF_CLEAR)
             return E_INVALIDARG;
-        memset(&decoder->output_format, 0, sizeof(decoder->output_format));
+        MoFreeMediaType(&decoder->output_type);
+        memset(&decoder->output_type, 0, sizeof(decoder->output_type));
         if (decoder->wg_transform)
         {
             wg_transform_destroy(decoder->wg_transform);
@@ -783,18 +821,14 @@ static HRESULT WINAPI media_object_SetOutputType(IMediaObject *iface, DWORD inde
     if (i == ARRAY_SIZE(wma_decoder_output_types))
         return DMO_E_TYPE_NOT_ACCEPTED;
 
-
-    if (!amt_to_wg_format((const AM_MEDIA_TYPE *)type, &wg_format))
-        return DMO_E_TYPE_NOT_ACCEPTED;
-    assert(wg_format.major_type == WG_MAJOR_TYPE_AUDIO);
-
-    if (decoder->input_format.major_type == WG_MAJOR_TYPE_UNKNOWN)
+    if (IsEqualGUID(&decoder->input_type.majortype, &GUID_NULL))
         return DMO_E_TYPE_NOT_SET;
-
     if (flags & DMO_SET_TYPEF_TEST_ONLY)
         return S_OK;
 
-    decoder->output_format = wg_format;
+    MoFreeMediaType(&decoder->output_type);
+    memset(&decoder->output_type, 0, sizeof(decoder->output_type));
+    MoCopyMediaType(&decoder->output_type, type);
 
     /* Set up wg_transform. */
     if (decoder->wg_transform)
@@ -802,22 +836,41 @@ static HRESULT WINAPI media_object_SetOutputType(IMediaObject *iface, DWORD inde
         wg_transform_destroy(decoder->wg_transform);
         decoder->wg_transform = 0;
     }
-    if (!(decoder->wg_transform = wg_transform_create(&decoder->input_format, &decoder->output_format, &attrs)))
-        return E_FAIL;
+    if (FAILED(hr = wg_transform_create_quartz(&decoder->input_type, &decoder->output_type,
+            &attrs, &decoder->wg_transform)))
+        return hr;
 
     return S_OK;
 }
 
 static HRESULT WINAPI media_object_GetInputCurrentType(IMediaObject *iface, DWORD index, DMO_MEDIA_TYPE *type)
 {
-    FIXME("iface %p, index %lu, type %p stub!\n", iface, index, type);
-    return E_NOTIMPL;
+    struct wma_decoder *decoder = impl_from_IMediaObject(iface);
+
+    TRACE("iface %p, index %lu, type %p\n", iface, index, type);
+
+    if (index)
+        return DMO_E_INVALIDSTREAMINDEX;
+    if (IsEqualGUID(&decoder->input_type.majortype, &GUID_NULL))
+        return DMO_E_TYPE_NOT_SET;
+    if (!type)
+        return E_POINTER;
+    return MoCopyMediaType(type, &decoder->input_type);
 }
 
 static HRESULT WINAPI media_object_GetOutputCurrentType(IMediaObject *iface, DWORD index, DMO_MEDIA_TYPE *type)
 {
-    FIXME("iface %p, index %lu, type %p stub!\n", iface, index, type);
-    return E_NOTIMPL;
+    struct wma_decoder *decoder = impl_from_IMediaObject(iface);
+
+    TRACE("iface %p, index %lu, type %p\n", iface, index, type);
+
+    if (index)
+        return DMO_E_INVALIDSTREAMINDEX;
+    if (IsEqualGUID(&decoder->output_type.majortype, &GUID_NULL))
+        return DMO_E_TYPE_NOT_SET;
+    if (!type)
+        return E_POINTER;
+    return MoCopyMediaType(type, &decoder->output_type);
 }
 
 static HRESULT WINAPI media_object_GetInputSizeInfo(IMediaObject *iface, DWORD index, DWORD *size,
@@ -838,7 +891,7 @@ static HRESULT WINAPI media_object_GetOutputSizeInfo(IMediaObject *iface, DWORD
         return E_POINTER;
     if (index > 0)
         return DMO_E_INVALIDSTREAMINDEX;
-    if (decoder->output_format.major_type == WG_MAJOR_TYPE_UNKNOWN)
+    if (IsEqualGUID(&decoder->output_type.majortype, &GUID_NULL))
         return DMO_E_TYPE_NOT_SET;
 
     *size = 8192;
@@ -866,6 +919,9 @@ static HRESULT WINAPI media_object_Flush(IMediaObject *iface)
 
     TRACE("iface %p.\n", iface);
 
+    if (!decoder->wg_transform)
+        return DMO_E_TYPE_NOT_SET;
+
     if (FAILED(hr = wg_transform_flush(decoder->wg_transform)))
         return hr;
 
@@ -936,6 +992,8 @@ static HRESULT WINAPI media_object_ProcessOutput(IMediaObject *iface, DWORD flag
         wg_sample_queue_flush(decoder->wg_sample_queue, false);
     }
 
+    if (hr == MF_E_TRANSFORM_NEED_MORE_INPUT)
+        return S_FALSE;
     return hr;
 }
 
