@@ -91,6 +91,7 @@ typedef LONG NTSTATUS;
 #define BCRYPT_DSA_ALGORITHM        L"DSA"
 #define BCRYPT_ECDH_P256_ALGORITHM  L"ECDH_P256"
 #define BCRYPT_ECDH_P384_ALGORITHM  L"ECDH_P384"
+#define BCRYPT_ECDH_P521_ALGORITHM  L"ECDH_P521"
 #define BCRYPT_ECDSA_P256_ALGORITHM L"ECDSA_P256"
 #define BCRYPT_ECDSA_P384_ALGORITHM L"ECDSA_P384"
 #define BCRYPT_ECDSA_P521_ALGORITHM L"ECDSA_P521"
@@ -106,6 +107,7 @@ typedef LONG NTSTATUS;
 #define BCRYPT_SHA256_ALGORITHM     L"SHA256"
 #define BCRYPT_SHA384_ALGORITHM     L"SHA384"
 #define BCRYPT_SHA512_ALGORITHM     L"SHA512"
+#define BCRYPT_PBKDF2_ALGORITHM     L"PBKDF2"
 
 #define BCRYPT_CHAIN_MODE_NA        L"ChainingModeN/A"
 #define BCRYPT_CHAIN_MODE_CBC       L"ChainingModeCBC"
@@ -189,6 +191,7 @@ static const WCHAR BCRYPT_SHA1_ALGORITHM[] = {'S','H','A','1',0};
 static const WCHAR BCRYPT_SHA256_ALGORITHM[] = {'S','H','A','2','5','6',0};
 static const WCHAR BCRYPT_SHA384_ALGORITHM[] = {'S','H','A','3','8','4',0};
 static const WCHAR BCRYPT_SHA512_ALGORITHM[] = {'S','H','A','5','1','2',0};
+static const WCHAR BCRYPT_PBKDF2_ALGORITHM[] = {'P','B','K','D','F','2',0};
 
 static const WCHAR BCRYPT_CHAIN_MODE_NA[] = {'C','h','a','i','n','i','n','g','M','o','d','e','N','/','A',0};
 static const WCHAR BCRYPT_CHAIN_MODE_CBC[] = {'C','h','a','i','n','i','n','g','M','o','d','e','C','B','C',0};
@@ -202,7 +205,8 @@ static const WCHAR BCRYPT_KDF_HMAC[] = {'H','M','A','C',0};
 static const WCHAR BCRYPT_KDF_TLS_PRF[] = {'T','L','S','_','P','R','F',0};
 static const WCHAR BCRYPT_KDF_SP80056A_CONCAT[] = {'S','P','8','0','0','_','5','6','A','_','C','O','N','C','A','T',0};
 static const WCHAR BCRYPT_KDF_RAW_SECRET[] = {'T','R','U','N','C','A','T','E',0};
-#define BCRYPT_DH_PARAMETERS        u"DHParameters"
+
+static const WCHAR BCRYPT_DH_PARAMETERS[] = {'D','H','P','a','r','a','m','e','t','e','r','s',0};
 #endif
 
 #define BCRYPT_ECDSA_PUBLIC_P256_MAGIC  0x31534345
@@ -225,6 +229,7 @@ static const WCHAR BCRYPT_KDF_RAW_SECRET[] = {'T','R','U','N','C','A','T','E',0}
 #define BCRYPT_SECRET_AGREEMENT_OPERATION       0x00000008
 #define BCRYPT_SIGNATURE_OPERATION              0x00000010
 #define BCRYPT_RNG_OPERATION                    0x00000020
+#define BCRYPT_KEY_DERIVATION_OPERATION         0x00000040
 
 #define BCRYPT_CIPHER_INTERFACE                 0x00000001
 #define BCRYPT_HASH_INTERFACE                   0x00000002
@@ -368,6 +373,15 @@ typedef struct _BCRYPT_DH_KEY_BLOB
     ULONG cbKey;
 } BCRYPT_DH_KEY_BLOB, *PBCRYPT_DH_KEY_BLOB;
 
+#define BCRYPT_DH_PARAMETERS_MAGIC  0x4d504844
+
+typedef struct _BCRYPT_DH_PARAMETER_HEADER
+{
+    ULONG cbLength;
+    ULONG dwMagic;
+    ULONG cbKeyLength;
+} BCRYPT_DH_PARAMETER_HEADER;
+
 #define BCRYPT_AUTHENTICATED_CIPHER_MODE_INFO_VERSION 1
 
 #define BCRYPT_AUTH_MODE_CHAIN_CALLS_FLAG 0x00000001
@@ -403,18 +417,23 @@ typedef struct _BCRYPT_KEY_DATA_BLOB_HEADER
     ULONG cbKeyData;
 } BCRYPT_KEY_DATA_BLOB_HEADER, *PBCRYPT_KEY_DATA_BLOB_HEADER;
 
-typedef struct _BCRYPT_DH_PARAMETER_HEADER
-{
-    ULONG cbLength;
-    ULONG dwMagic;
-    ULONG cbKeyLength;
-} BCRYPT_DH_PARAMETER_HEADER;
-
-#define BCRYPT_DH_PARAMETERS_MAGIC 0x4d504844
-
-#define KDF_HASH_ALGORITHM 0x00000000
-#define KDF_SECRET_PREPEND 0x00000001
-#define KDF_SECRET_APPEND  0x00000002
+#define KDF_HASH_ALGORITHM      0x00
+#define KDF_SECRET_PREPEND      0x01
+#define KDF_SECRET_APPEND       0x02
+#define KDF_HMAC_KEY            0x03
+#define KDF_TLS_PRF_LABEL       0x04
+#define KDF_TLS_PRF_SEED        0x05
+#define KDF_SECRET_HANDLE       0x06
+#define KDF_TLS_PRF_PROTOCOL    0x07
+#define KDF_ALGORITHMID         0x08
+#define KDF_PARTYUINFO          0x09
+#define KDF_PARTYVINFO          0x0a
+#define KDF_SUPPPUBINFO         0x0b
+#define KDF_SUPPPRIVINFO        0x0c
+#define KDF_LABEL               0x0d
+#define KDF_CONTEXT             0x0e
+#define KDF_SALT                0x0f
+#define KDF_ITERATION_COUNT     0x10
 
 typedef struct _BCryptBuffer
 {
@@ -551,6 +570,7 @@ NTSTATUS WINAPI BCryptHash(BCRYPT_ALG_HANDLE, PUCHAR, ULONG, PUCHAR, ULONG, PUCH
 NTSTATUS WINAPI BCryptHashData(BCRYPT_HASH_HANDLE, PUCHAR, ULONG, ULONG);
 NTSTATUS WINAPI BCryptImportKey(BCRYPT_ALG_HANDLE, BCRYPT_KEY_HANDLE, LPCWSTR, BCRYPT_KEY_HANDLE *, PUCHAR, ULONG, PUCHAR, ULONG, ULONG);
 NTSTATUS WINAPI BCryptImportKeyPair(BCRYPT_ALG_HANDLE, BCRYPT_KEY_HANDLE, LPCWSTR, BCRYPT_KEY_HANDLE *, UCHAR *, ULONG, ULONG);
+NTSTATUS WINAPI BCryptKeyDerivation(BCRYPT_KEY_HANDLE, BCryptBufferDesc *, UCHAR *, ULONG, ULONG *, ULONG);
 NTSTATUS WINAPI BCryptOpenAlgorithmProvider(BCRYPT_ALG_HANDLE *, LPCWSTR, LPCWSTR, ULONG);
 NTSTATUS WINAPI BCryptRemoveContextFunction(ULONG, LPCWSTR, ULONG, LPCWSTR);
 NTSTATUS WINAPI BCryptSecretAgreement(BCRYPT_KEY_HANDLE, BCRYPT_KEY_HANDLE, BCRYPT_SECRET_HANDLE *, ULONG);
