@@ -378,6 +378,15 @@ static UINT64 read_tsc_frequency(void)
     return freq;
 }
 
+#elif defined(__aarch64__)
+
+static UINT64 read_tsc_frequency(void)
+{
+    UINT64 tsc_frequency;
+    __asm__ volatile( "mrs %[Res], CNTFRQ_EL0" : [Res] "=r" (tsc_frequency) );
+    return tsc_frequency;
+}
+
 #else
 
 static void initialize_xstate_features(struct _KUSER_SHARED_DATA *data)
@@ -469,13 +478,30 @@ static void create_user_shared_data(void)
     case PROCESSOR_ARCHITECTURE_ARM:
         features[PF_ARM_VFP_32_REGISTERS_AVAILABLE]       = !!(sci.ProcessorFeatureBits & CPU_FEATURE_ARM_VFP_32);
         features[PF_ARM_NEON_INSTRUCTIONS_AVAILABLE]      = !!(sci.ProcessorFeatureBits & CPU_FEATURE_ARM_NEON);
-        features[PF_ARM_V8_INSTRUCTIONS_AVAILABLE]        = (sci.ProcessorLevel >= 8);
         break;
 
     case PROCESSOR_ARCHITECTURE_ARM64:
         features[PF_ARM_V8_INSTRUCTIONS_AVAILABLE]        = TRUE;
         features[PF_ARM_V8_CRC32_INSTRUCTIONS_AVAILABLE]  = !!(sci.ProcessorFeatureBits & CPU_FEATURE_ARM_V8_CRC32);
         features[PF_ARM_V8_CRYPTO_INSTRUCTIONS_AVAILABLE] = !!(sci.ProcessorFeatureBits & CPU_FEATURE_ARM_V8_CRYPTO);
+        features[PF_ARM_V81_ATOMIC_INSTRUCTIONS_AVAILABLE]= !!(sci.ProcessorFeatureBits & CPU_FEATURE_ARM_V81_ATOMIC);
+        features[PF_ARM_V82_DP_INSTRUCTIONS_AVAILABLE]    = !!(sci.ProcessorFeatureBits & CPU_FEATURE_ARM_V82_DP);
+        features[PF_ARM_V83_JSCVT_INSTRUCTIONS_AVAILABLE] = !!(sci.ProcessorFeatureBits & CPU_FEATURE_ARM_V83_JSCVT);
+        features[PF_ARM_V83_LRCPC_INSTRUCTIONS_AVAILABLE] = !!(sci.ProcessorFeatureBits & CPU_FEATURE_ARM_V83_LRCPC);
+        features[PF_ARM_SVE_INSTRUCTIONS_AVAILABLE]       = !!(sci.ProcessorFeatureBits & CPU_FEATURE_ARM_SVE);
+        features[PF_ARM_SVE2_INSTRUCTIONS_AVAILABLE]      = !!(sci.ProcessorFeatureBits & CPU_FEATURE_ARM_SVE2);
+        features[PF_ARM_SVE2_1_INSTRUCTIONS_AVAILABLE]    = !!(sci.ProcessorFeatureBits & CPU_FEATURE_ARM_SVE2_1);
+        features[PF_ARM_SVE_AES_INSTRUCTIONS_AVAILABLE]   = !!(sci.ProcessorFeatureBits & CPU_FEATURE_ARM_SVE_AES);
+        features[PF_ARM_SVE_PMULL128_INSTRUCTIONS_AVAILABLE] = !!(sci.ProcessorFeatureBits & CPU_FEATURE_ARM_SVE_PMULL128);
+        features[PF_ARM_SVE_BITPERM_INSTRUCTIONS_AVAILABLE] = !!(sci.ProcessorFeatureBits & CPU_FEATURE_ARM_SVE_BITPERM);
+        features[PF_ARM_SVE_BF16_INSTRUCTIONS_AVAILABLE]  = !!(sci.ProcessorFeatureBits & CPU_FEATURE_ARM_SVE_BF16);
+        features[PF_ARM_SVE_EBF16_INSTRUCTIONS_AVAILABLE] = !!(sci.ProcessorFeatureBits & CPU_FEATURE_ARM_SVE_EBF16);
+        features[PF_ARM_SVE_B16B16_INSTRUCTIONS_AVAILABLE] = !!(sci.ProcessorFeatureBits & CPU_FEATURE_ARM_SVE_B16B16);
+        features[PF_ARM_SVE_SHA3_INSTRUCTIONS_AVAILABLE]  = !!(sci.ProcessorFeatureBits & CPU_FEATURE_ARM_SVE_SHA3);
+        features[PF_ARM_SVE_SM4_INSTRUCTIONS_AVAILABLE]   = !!(sci.ProcessorFeatureBits & CPU_FEATURE_ARM_SVE_SM4);
+        features[PF_ARM_SVE_I8MM_INSTRUCTIONS_AVAILABLE]  = !!(sci.ProcessorFeatureBits & CPU_FEATURE_ARM_SVE_I8MM);
+        features[PF_ARM_SVE_F32MM_INSTRUCTIONS_AVAILABLE] = !!(sci.ProcessorFeatureBits & CPU_FEATURE_ARM_SVE_F32MM);
+        features[PF_ARM_SVE_F64MM_INSTRUCTIONS_AVAILABLE] = !!(sci.ProcessorFeatureBits & CPU_FEATURE_ARM_SVE_F64MM);
         features[PF_COMPARE_EXCHANGE_DOUBLE]              = TRUE;
         features[PF_NX_ENABLED]                           = TRUE;
         features[PF_FASTFAIL_AVAILABLE]                   = TRUE;
@@ -515,60 +541,6 @@ static void create_user_shared_data(void)
     UnmapViewOfFile( data );
 }
 
-#if defined(__i386__) || defined(__x86_64__)
-
-static void regs_to_str( int *regs, unsigned int len, WCHAR *buffer )
-{
-    unsigned int i;
-    unsigned char *p = (unsigned char *)regs;
-
-    for (i = 0; i < len; i++) { buffer[i] = *p++; }
-    buffer[i] = 0;
-}
-
-static unsigned int get_model( unsigned int reg0, unsigned int *stepping, unsigned int *family )
-{
-    unsigned int model, family_id = (reg0 & (0x0f << 8)) >> 8;
-
-    model = (reg0 & (0x0f << 4)) >> 4;
-    if (family_id == 6 || family_id == 15) model |= (reg0 & (0x0f << 16)) >> 12;
-
-    *family = family_id;
-    if (family_id == 15) *family += (reg0 & (0xff << 20)) >> 20;
-
-    *stepping = reg0 & 0x0f;
-    return model;
-}
-
-static void get_identifier( WCHAR *buf, size_t size, const WCHAR *arch )
-{
-    unsigned int family, model, stepping;
-    int regs[4] = {0, 0, 0, 0};
-
-    __cpuid( regs, 1 );
-    model = get_model( regs[0], &stepping, &family );
-    swprintf( buf, size, L"%s Family %u Model %u Stepping %u", arch, family, model, stepping );
-}
-
-static void get_vendorid( WCHAR *buf )
-{
-    int tmp, regs[4] = {0, 0, 0, 0};
-
-    __cpuid( regs, 0 );
-    tmp = regs[2];      /* swap edx and ecx */
-    regs[2] = regs[3];
-    regs[3] = tmp;
-
-    regs_to_str( regs + 1, 12, buf );
-}
-
-#else  /* __i386__ || __x86_64__ */
-
-static void get_identifier( WCHAR *buf, size_t size, const WCHAR *arch ) { }
-static void get_vendorid( WCHAR *buf ) { }
-
-#endif  /* __i386__ || __x86_64__ */
-
 #include "pshpack1.h"
 struct smbios_prologue
 {
@@ -581,9 +553,14 @@ struct smbios_prologue
 
 enum smbios_type
 {
-    SMBIOS_TYPE_BIOS,
-    SMBIOS_TYPE_SYSTEM,
-    SMBIOS_TYPE_BASEBOARD,
+    SMBIOS_TYPE_BIOS = 0,
+    SMBIOS_TYPE_SYSTEM = 1,
+    SMBIOS_TYPE_BASEBOARD = 2,
+    SMBIOS_TYPE_CHASSIS = 3,
+    SMBIOS_TYPE_PROCESSOR = 4,
+    SMBIOS_TYPE_BOOTINFO = 32,
+    SMBIOS_TYPE_PROCESSOR_ADDITIONAL_INFO = 44,
+    SMBIOS_TYPE_END = 127
 };
 
 struct smbios_header
@@ -630,11 +607,68 @@ struct smbios_system
     BYTE                 sku;
     BYTE                 family;
 };
+
+struct smbios_processor
+{
+    struct smbios_header hdr;
+    BYTE                 socket;
+    BYTE                 type;
+    BYTE                 family;
+    BYTE                 vendor;
+    ULONGLONG            id;
+    BYTE                 version;
+    BYTE                 voltage;
+    WORD                 clock;
+    WORD                 max_speed;
+    WORD                 cur_speed;
+    BYTE                 status;
+    BYTE                 upgrade;
+    WORD                 l1cache;
+    WORD                 l2cache;
+    WORD                 l3cache;
+    BYTE                 serial;
+    BYTE                 asset_tag;
+    BYTE                 part_number;
+    BYTE                 core_count;
+    BYTE                 core_enabled;
+    BYTE                 thread_count;
+    WORD                 characteristics;
+    WORD                 family2;
+    WORD                 core_count2;
+    WORD                 core_enabled2;
+    WORD                 thread_count2;
+};
+
+struct smbios_processor_specific_block
+{
+    BYTE length;
+    BYTE processor_type;
+    BYTE data[];
+};
+
+struct smbios_processor_additional_info
+{
+    struct smbios_header hdr;
+    WORD ref_handle;
+    struct smbios_processor_specific_block info_block;
+};
+
+struct smbios_wine_core_id_regs_arm64
+{
+    WORD num_regs;
+    struct smbios_wine_id_reg_value_arm64
+    {
+        WORD reg;
+        UINT64 value;
+    } regs[];
+};
+
 #include "poppack.h"
 
 #define RSMB (('R' << 24) | ('S' << 16) | ('M' << 8) | 'B')
 
-static const struct smbios_header *find_smbios_entry( enum smbios_type type, const char *buf, UINT len )
+static const struct smbios_header *find_smbios_entry( enum smbios_type type, unsigned int index,
+                                                      const char *buf, UINT len )
 {
     const char *ptr, *start;
     const struct smbios_prologue *prologue;
@@ -660,20 +694,16 @@ static const struct smbios_header *find_smbios_entry( enum smbios_type type, con
         if (hdr->type == type)
         {
             if ((const char *)hdr - start + hdr->length > prologue->length) return NULL;
-            break;
+            if (!index--) return hdr;
         }
-        else /* skip other entries and their strings */
+        /* skip other entries and their strings */
+        for (ptr = (const char *)hdr + hdr->length; ptr - buf < len && *ptr; ptr++)
         {
-            for (ptr = (const char *)hdr + hdr->length; ptr - buf < len && *ptr; ptr++)
-            {
-                for (; ptr - buf < len; ptr++) if (!*ptr) break;
-            }
-            if (ptr == (const char *)hdr + hdr->length) ptr++;
-            hdr = (const struct smbios_header *)(ptr + 1);
+            for (; ptr - buf < len; ptr++) if (!*ptr) break;
         }
+        if (ptr == (const char *)hdr + hdr->length) ptr++;
+        hdr = (const struct smbios_header *)(ptr + 1);
     }
-
-    return hdr;
 }
 
 static inline WCHAR *heap_strdupAW( const char *src )
@@ -714,7 +744,7 @@ static void create_bios_baseboard_values( HKEY bios_key, const char *buf, UINT l
     const struct smbios_baseboard *baseboard;
     UINT offset;
 
-    if (!(hdr = find_smbios_entry( SMBIOS_TYPE_BASEBOARD, buf, len ))) return;
+    if (!(hdr = find_smbios_entry( SMBIOS_TYPE_BASEBOARD, 0, buf, len ))) return;
     baseboard = (const struct smbios_baseboard *)hdr;
     offset = (const char *)baseboard - buf + baseboard->hdr.length;
 
@@ -729,7 +759,7 @@ static void create_bios_bios_values( HKEY bios_key, const char *buf, UINT len )
     const struct smbios_bios *bios;
     UINT offset;
 
-    if (!(hdr = find_smbios_entry( SMBIOS_TYPE_BIOS, buf, len ))) return;
+    if (!(hdr = find_smbios_entry( SMBIOS_TYPE_BIOS, 0, buf, len ))) return;
     bios = (const struct smbios_bios *)hdr;
     offset = (const char *)bios - buf + bios->hdr.length;
 
@@ -759,7 +789,7 @@ static void create_bios_system_values( HKEY bios_key, const char *buf, UINT len
     const struct smbios_system *system;
     UINT offset;
 
-    if (!(hdr = find_smbios_entry( SMBIOS_TYPE_SYSTEM, buf, len ))) return;
+    if (!(hdr = find_smbios_entry( SMBIOS_TYPE_SYSTEM, 0, buf, len ))) return;
     system = (const struct smbios_system *)hdr;
     offset = (const char *)system - buf + system->hdr.length;
 
@@ -779,45 +809,46 @@ static void create_bios_system_values( HKEY bios_key, const char *buf, UINT len
     }
 }
 
-static void create_bios_key( HKEY system_key )
+#ifdef __aarch64__
+
+static void create_id_reg_keys_arm64( HKEY core_key, UINT core, const char *buf, UINT len )
 {
-    HKEY bios_key;
-    UINT len;
-    char *buf;
+    const struct smbios_header *hdr;
+    const struct smbios_processor_additional_info *additional_info;
+    const struct smbios_wine_core_id_regs_arm64 *core_id_regs;
+    static const char *reg_value_name = "CP %04X";
+    char buffer[9];
+    UINT i;
 
-    if (RegCreateKeyExW( system_key, L"BIOS", 0, NULL, REG_OPTION_VOLATILE,
-                         KEY_ALL_ACCESS, NULL, &bios_key, NULL ))
-        return;
+    if (!(hdr = find_smbios_entry( SMBIOS_TYPE_PROCESSOR_ADDITIONAL_INFO, core, buf, len ))) return;
 
-    len = GetSystemFirmwareTable( RSMB, 0, NULL, 0 );
-    if (!(buf = malloc( len ))) goto done;
-    len = GetSystemFirmwareTable( RSMB, 0, buf, len );
+    additional_info = (const struct smbios_processor_additional_info *)hdr;
+    core_id_regs = (const struct smbios_wine_core_id_regs_arm64 *)additional_info->info_block.data;
 
-    create_bios_baseboard_values( bios_key, buf, len );
-    create_bios_bios_values( bios_key, buf, len );
-    create_bios_system_values( bios_key, buf, len );
-
-done:
-    free( buf );
-    RegCloseKey( bios_key );
+    for (i = 0; i < core_id_regs->num_regs; i++)
+    {
+        snprintf( buffer, sizeof(buffer), reg_value_name, core_id_regs->regs[i].reg );
+        RegSetValueExA( core_key, buffer, 0, REG_QWORD,
+                        (const BYTE *)&core_id_regs->regs[i].value, sizeof(UINT64) );
+    }
 }
 
-/* create the volatile hardware registry keys */
-static void create_hardware_registry_keys(void)
+#endif
+
+static void create_bios_processor_values( HKEY system_key, const char *buf, UINT len )
 {
-    unsigned int i;
-    HKEY hkey, system_key, cpu_key, fpu_key;
+    const struct smbios_header *hdr;
+    const struct smbios_processor *proc;
+    unsigned int pkg, core, offset, i, thread_count;
+    HKEY hkey, cpu_key, fpu_key = 0, env_key;
     SYSTEM_CPU_INFORMATION sci;
     PROCESSOR_POWER_INFORMATION* power_info;
     ULONG sizeof_power_info = sizeof(PROCESSOR_POWER_INFORMATION) * NtCurrentTeb()->Peb->NumberOfProcessors;
     UINT64 tsc_frequency = read_tsc_frequency();
-    ULONG name_buffer[16];
-    WCHAR id[60], vendorid[13];
+    const WCHAR *arch;
+    WCHAR id[60], buffer[128], *version, *vendorid;
 
-    get_vendorid( vendorid );
     NtQuerySystemInformation( SystemCpuInformation, &sci, sizeof(sci), NULL );
-    if (NtQuerySystemInformation( SystemProcessorBrandString, name_buffer, sizeof(name_buffer), NULL ))
-        name_buffer[0] = 0;
 
     power_info = malloc( sizeof_power_info );
     if (power_info == NULL)
@@ -825,31 +856,6 @@ static void create_hardware_registry_keys(void)
     if (NtPowerInformation( ProcessorInformation, NULL, 0, power_info, sizeof_power_info ))
         memset( power_info, 0, sizeof_power_info );
 
-    switch (sci.ProcessorArchitecture)
-    {
-    case PROCESSOR_ARCHITECTURE_ARM:
-    case PROCESSOR_ARCHITECTURE_ARM64:
-        swprintf( id, ARRAY_SIZE(id), L"ARM Family %u Model %u Revision %u",
-                  sci.ProcessorLevel, HIBYTE(sci.ProcessorRevision), LOBYTE(sci.ProcessorRevision) );
-        break;
-
-    case PROCESSOR_ARCHITECTURE_AMD64:
-        get_identifier( id, ARRAY_SIZE(id), !wcscmp(vendorid, L"AuthenticAMD") ? L"AMD64" : L"Intel64" );
-        break;
-
-    case PROCESSOR_ARCHITECTURE_INTEL:
-    default:
-        get_identifier( id, ARRAY_SIZE(id), L"x86" );
-        break;
-    }
-
-    if (RegCreateKeyExW( HKEY_LOCAL_MACHINE, L"Hardware\\Description\\System", 0, NULL,
-                         REG_OPTION_VOLATILE, KEY_ALL_ACCESS, NULL, &system_key, NULL ))
-    {
-        free( power_info );
-        return;
-    }
-
     switch (sci.ProcessorArchitecture)
     {
     case PROCESSOR_ARCHITECTURE_ARM:
@@ -861,56 +867,139 @@ static void create_hardware_registry_keys(void)
     case PROCESSOR_ARCHITECTURE_AMD64:
     default:
         set_reg_value( system_key, L"Identifier", L"AT compatible" );
+        if (RegCreateKeyExW( system_key, L"FloatingPointProcessor", 0, NULL, REG_OPTION_VOLATILE,
+                             KEY_ALL_ACCESS, NULL, &fpu_key, NULL ))
+            fpu_key = 0;
         break;
     }
 
-    if (sci.ProcessorArchitecture == PROCESSOR_ARCHITECTURE_ARM ||
-        sci.ProcessorArchitecture == PROCESSOR_ARCHITECTURE_ARM64 ||
-        RegCreateKeyExW( system_key, L"FloatingPointProcessor", 0, NULL, REG_OPTION_VOLATILE,
-                         KEY_ALL_ACCESS, NULL, &fpu_key, NULL ))
-        fpu_key = 0;
     if (RegCreateKeyExW( system_key, L"CentralProcessor", 0, NULL, REG_OPTION_VOLATILE,
                          KEY_ALL_ACCESS, NULL, &cpu_key, NULL ))
         cpu_key = 0;
 
-    for (i = 0; i < NtCurrentTeb()->Peb->NumberOfProcessors; i++)
+    for (pkg = core = 0; ; pkg++)
     {
-        WCHAR numW[10];
+        if (!(hdr = find_smbios_entry( SMBIOS_TYPE_PROCESSOR, pkg, buf, len ))) break;
+        if (hdr->length < 0x28) break; /* version 2.5 is required */
+        proc = (const struct smbios_processor *)hdr;
+        offset = (const char *)proc - buf + proc->hdr.length;
+        version = get_smbios_string( proc->version, buf, offset, len );
+        vendorid = get_smbios_string( proc->vendor, buf, offset, len );
 
-        swprintf( numW, ARRAY_SIZE(numW), L"%u", i );
-        if (!RegCreateKeyExW( cpu_key, numW, 0, NULL, REG_OPTION_VOLATILE,
-                              KEY_ALL_ACCESS, NULL, &hkey, NULL ))
+        switch (sci.ProcessorArchitecture)
         {
-            DWORD tsc_freq_mhz = (DWORD)(tsc_frequency / 1000000ull); /* Hz -> Mhz */
-            if (!tsc_freq_mhz) tsc_freq_mhz = power_info[i].MaxMhz;
+        case PROCESSOR_ARCHITECTURE_ARM:
+            arch = L"ARM";
+            swprintf( id, ARRAY_SIZE(id), L"ARM Family 7 Model %X Revision %X",
+                      sci.ProcessorLevel, sci.ProcessorRevision );
+            break;
+        case PROCESSOR_ARCHITECTURE_ARM64:
+            arch = L"ARM64";
+            swprintf( id, ARRAY_SIZE(id), L"ARMv8 (64-bit) Family 8 Model %X Revision %X",
+                      sci.ProcessorLevel, sci.ProcessorRevision );
+            break;
+        case PROCESSOR_ARCHITECTURE_AMD64:
+            arch = L"AMD64";
+            swprintf( id, ARRAY_SIZE(id), L"%s Family %u Model %u Stepping %u",
+                      vendorid && !wcscmp(vendorid, L"AuthenticAMD") ? L"AMD64" : L"Intel64",
+                      sci.ProcessorLevel, HIBYTE(sci.ProcessorRevision), LOBYTE(sci.ProcessorRevision) );
+            break;
+        case PROCESSOR_ARCHITECTURE_INTEL:
+        default:
+            arch = L"x86";
+            swprintf( id, ARRAY_SIZE(id), L"x86 Family %u Model %u Stepping %u",
+                      sci.ProcessorLevel, HIBYTE(sci.ProcessorRevision), LOBYTE(sci.ProcessorRevision) );
+            break;
+        }
 
-            RegSetValueExW( hkey, L"FeatureSet", 0, REG_DWORD, (BYTE *)&sci.ProcessorFeatureBits, sizeof(DWORD) );
-            set_reg_value( hkey, L"Identifier", id );
-            /* TODO: report ARM properly */
-            RegSetValueExA( hkey, "ProcessorNameString", 0, REG_SZ, (const BYTE *)name_buffer,
-                            strlen( (char *)name_buffer ) + 1 );
-            set_reg_value( hkey, L"VendorIdentifier", vendorid );
-            RegSetValueExW( hkey, L"~MHz", 0, REG_DWORD, (BYTE *)&tsc_freq_mhz, sizeof(DWORD) );
-            RegCloseKey( hkey );
-        }
-        if (sci.ProcessorArchitecture != PROCESSOR_ARCHITECTURE_ARM &&
-            sci.ProcessorArchitecture != PROCESSOR_ARCHITECTURE_ARM64 &&
-            !RegCreateKeyExW( fpu_key, numW, 0, NULL, REG_OPTION_VOLATILE,
-                              KEY_ALL_ACCESS, NULL, &hkey, NULL ))
+        thread_count = (proc->hdr.length >= 0x30) ? proc->thread_count2 : proc->thread_count;
+        for (i = 0; i < thread_count; i++, core++)
         {
-            set_reg_value( hkey, L"Identifier", id );
-            RegCloseKey( hkey );
+            swprintf( buffer, ARRAY_SIZE(buffer), L"%u", core );
+            if (!RegCreateKeyExW( cpu_key, buffer, 0, NULL, REG_OPTION_VOLATILE,
+                                  KEY_ALL_ACCESS, NULL, &hkey, NULL ))
+            {
+                DWORD tsc_freq_mhz = (DWORD)(tsc_frequency / 1000000ull); /* Hz -> Mhz */
+                if (!tsc_freq_mhz) tsc_freq_mhz = power_info[core].MaxMhz;
+
+                RegSetValueExW( hkey, L"FeatureSet", 0, REG_DWORD,
+                                (BYTE *)&sci.ProcessorFeatureBits, sizeof(DWORD) );
+                set_reg_value( hkey, L"Identifier", id );
+                if (vendorid) set_reg_value( hkey, L"VendorIdentifier", vendorid );
+                if (version) set_reg_value( hkey, L"ProcessorNameString", version );
+                RegSetValueExW( hkey, L"~MHz", 0, REG_DWORD, (BYTE *)&tsc_freq_mhz, sizeof(DWORD) );
+#ifdef __aarch64__
+                create_id_reg_keys_arm64( hkey, core, buf, len );
+#endif
+                RegCloseKey( hkey );
+            }
+            if (fpu_key && !RegCreateKeyExW( fpu_key, buffer, 0, NULL, REG_OPTION_VOLATILE,
+                                             KEY_ALL_ACCESS, NULL, &hkey, NULL ))
+            {
+                set_reg_value( hkey, L"Identifier", id );
+                RegCloseKey( hkey );
+            }
         }
+
+        if (!pkg && !RegCreateKeyW( HKEY_LOCAL_MACHINE,
+                                    L"System\\CurrentControlSet\\Control\\Session Manager\\Environment",
+                                    &env_key ))
+        {
+            set_reg_value( env_key, L"PROCESSOR_ARCHITECTURE", arch );
+
+            swprintf( buffer, ARRAY_SIZE(buffer), L"%s, %s", id, vendorid ? vendorid : L"Unknown" );
+            set_reg_value( env_key, L"PROCESSOR_IDENTIFIER", buffer );
+
+            swprintf( buffer, ARRAY_SIZE(buffer), L"%u", sci.ProcessorLevel );
+            set_reg_value( env_key, L"PROCESSOR_LEVEL", buffer );
+
+            swprintf( buffer, ARRAY_SIZE(buffer), L"%04x", sci.ProcessorRevision );
+            set_reg_value( env_key, L"PROCESSOR_REVISION", buffer );
+
+            swprintf( buffer, ARRAY_SIZE(buffer), L"%u", NtCurrentTeb()->Peb->NumberOfProcessors );
+            set_reg_value( env_key, L"NUMBER_OF_PROCESSORS", buffer );
+            RegCloseKey( env_key );
+        }
+
+        free( version );
+        free( vendorid );
     }
 
-    create_bios_key( system_key );
-
     RegCloseKey( fpu_key );
     RegCloseKey( cpu_key );
-    RegCloseKey( system_key );
     free( power_info );
 }
 
+/* create the volatile hardware registry keys */
+static void create_hardware_registry_keys(void)
+{
+    HKEY system_key, bios_key;
+    UINT len;
+    char *buf;
+
+    len = GetSystemFirmwareTable( RSMB, 0, NULL, 0 );
+    if (!(buf = malloc( len ))) return;
+    len = GetSystemFirmwareTable( RSMB, 0, buf, len );
+
+    if (RegCreateKeyExW( HKEY_LOCAL_MACHINE, L"Hardware\\Description\\System", 0, NULL,
+                         REG_OPTION_VOLATILE, KEY_ALL_ACCESS, NULL, &system_key, NULL ))
+    {
+        free( buf );
+        return;
+    }
+    if (!RegCreateKeyExW( system_key, L"BIOS", 0, NULL, REG_OPTION_VOLATILE,
+                          KEY_ALL_ACCESS, NULL, &bios_key, NULL ))
+    {
+        create_bios_baseboard_values( bios_key, buf, len );
+        create_bios_bios_values( bios_key, buf, len );
+        create_bios_system_values( bios_key, buf, len );
+        RegCloseKey( bios_key );
+    }
+    create_bios_processor_values( system_key, buf, len );
+    RegCloseKey( system_key );
+    free( buf );
+}
+
 
 /* create the DynData registry keys */
 static void create_dynamic_registry_keys(void)
@@ -923,63 +1012,6 @@ static void create_dynamic_registry_keys(void)
         RegCloseKey( key );
 }
 
-/* create the platform-specific environment registry keys */
-static void create_environment_registry_keys( void )
-{
-    HKEY env_key;
-    SYSTEM_CPU_INFORMATION sci;
-    WCHAR buffer[60], vendorid[13];
-    const WCHAR *arch, *parch;
-
-    if (RegCreateKeyW( HKEY_LOCAL_MACHINE, L"System\\CurrentControlSet\\Control\\Session Manager\\Environment", &env_key )) return;
-
-    get_vendorid( vendorid );
-    NtQuerySystemInformation( SystemCpuInformation, &sci, sizeof(sci), NULL );
-
-    swprintf( buffer, ARRAY_SIZE(buffer), L"%u", NtCurrentTeb()->Peb->NumberOfProcessors );
-    set_reg_value( env_key, L"NUMBER_OF_PROCESSORS", buffer );
-
-    switch (sci.ProcessorArchitecture)
-    {
-    case PROCESSOR_ARCHITECTURE_AMD64:
-        arch = L"AMD64";
-        parch = !wcscmp(vendorid, L"AuthenticAMD") ? L"AMD64" : L"Intel64";
-        break;
-
-    case PROCESSOR_ARCHITECTURE_INTEL:
-    default:
-        arch = parch = L"x86";
-        break;
-    }
-    set_reg_value( env_key, L"PROCESSOR_ARCHITECTURE", arch );
-
-    switch (sci.ProcessorArchitecture)
-    {
-    case PROCESSOR_ARCHITECTURE_ARM:
-    case PROCESSOR_ARCHITECTURE_ARM64:
-        swprintf( buffer, ARRAY_SIZE(buffer), L"ARM Family %u Model %u Revision %u",
-                  sci.ProcessorLevel, HIBYTE(sci.ProcessorRevision), LOBYTE(sci.ProcessorRevision) );
-        break;
-
-    case PROCESSOR_ARCHITECTURE_AMD64:
-    case PROCESSOR_ARCHITECTURE_INTEL:
-    default:
-        get_identifier( buffer, ARRAY_SIZE(buffer), parch );
-        lstrcatW( buffer, L", " );
-        lstrcatW( buffer, vendorid );
-        break;
-    }
-    set_reg_value( env_key, L"PROCESSOR_IDENTIFIER", buffer );
-
-    swprintf( buffer, ARRAY_SIZE(buffer), L"%u", sci.ProcessorLevel );
-    set_reg_value( env_key, L"PROCESSOR_LEVEL", buffer );
-
-    swprintf( buffer, ARRAY_SIZE(buffer), L"%04x", sci.ProcessorRevision );
-    set_reg_value( env_key, L"PROCESSOR_REVISION", buffer );
-
-    RegCloseKey( env_key );
-}
-
 /* create the ComputerName registry keys */
 static void create_computer_name_keys(void)
 {
@@ -1020,6 +1052,14 @@ static void create_computer_name_keys(void)
 
 static void create_volatile_environment_registry_key(void)
 {
+    static const WCHAR *preserve[] =
+    {
+        L"DXVK_ENABLE_NVAPI",
+        L"DXVK_NVAPI_ALLOW_OTHER_DRIVERS",
+        L"DXVK_NVAPI_DRIVER_VERSION",
+    };
+    const WCHAR *str;
+    unsigned int i;
     WCHAR path[MAX_PATH];
     WCHAR computername[MAX_COMPUTERNAME_LENGTH + 1 + 2];
     DWORD size;
@@ -1064,6 +1104,12 @@ static void create_volatile_environment_registry_key(void)
     }
 
     set_reg_value( hkey, L"SESSIONNAME", L"Console" );
+
+    for (i = 0; i < ARRAY_SIZE(preserve); ++i)
+    {
+        if ((str = _wgetenv( preserve[i] ))) set_reg_value( hkey, preserve[i], str );
+    }
+
     RegCloseKey( hkey );
 }
 
@@ -1500,6 +1546,7 @@ static void install_root_pnp_devices(void)
     }
     root_devices[] =
     {
+        {"root\\wine\\winebth", "root\\winebth\0", "C:\\windows\\inf\\winebth.inf"},
         {"root\\wine\\winebus", "root\\winebus\0", "C:\\windows\\inf\\winebus.inf"},
         {"root\\wine\\wineusb", "root\\wineusb\0", "C:\\windows\\inf\\wineusb.inf"},
     };
@@ -1523,7 +1570,7 @@ static void install_root_pnp_devices(void)
         }
 
         if (!SetupDiSetDeviceRegistryPropertyA(set, &device, SPDRP_HARDWAREID,
-                (const BYTE *)root_devices[i].hardware_id, (strlen(root_devices[i].hardware_id) + 2) * sizeof(WCHAR)))
+                (const BYTE *)root_devices[i].hardware_id, strlen(root_devices[i].hardware_id) + 2))
         {
             WINE_ERR("Failed to set hardware id for %s, error %#lx.\n", debugstr_a(root_devices[i].name), GetLastError());
             continue;
@@ -1580,7 +1627,7 @@ static void update_user_profile(void)
 
 static void update_win_version(void)
 {
-    static const WCHAR win10_buildW[] = L"19043";
+    static const WCHAR win10_buildW[] = L"19045";
     static const WCHAR win10_ntW[] = L"6.3";
 
     HKEY cv_h;
@@ -1686,7 +1733,7 @@ static void update_wineprefix( BOOL force )
         update_win_version();
         update_root_certs();
 
-        WINE_MESSAGE( "wine: configuration in %s has been updated.\n", debugstr_w(prettyprint_configdir()) );
+        TRACE( "wine: configuration in %s has been updated.\n", debugstr_w(prettyprint_configdir()) );
     }
 
 done:
@@ -1927,7 +1974,6 @@ int __cdecl main( int argc, char *argv[] )
     create_user_shared_data();
     create_hardware_registry_keys();
     create_dynamic_registry_keys();
-    create_environment_registry_keys();
     create_computer_name_keys();
     wininit();
     pendingRename();
